function ZhLogger(templateManager, ZHPromise, ScreenService, drivers) {//, requestEngine you cant

	var enabledModes = [],
		syncRoute = "",
		defaultModes = ["debug", "info", "warning", "error", "fatal"];
	
	templateManager.projectPromise.then(function() {
		if (templateManager.project.logger && Util.isString(templateManager.project.logger.syncRoute)) {
			syncRoute = templateManager.project.logger.syncRoute;
		}
		if (templateManager.project.logger && Util.isArray(templateManager.project.logger.enabledModes)) {
			enabledModes = Util.clone(templateManager.project.logger.enabledModes);
		} else {
			enabledModes = Util.clone(defaultModes);
		}
	});


	function log(mode, message, objectArray, tag, throwable) {
		if (enabledModes.indexOf(mode) !== -1) {
			message = "[" + mode + "]" + (tag ? ("[" + tag + "]") : "") + " " + message;
			if (Util.isGenericObject(throwable)) {
				message += "\n" + throwable.name + ": " + throwable.message + ". Stack: " + throwable.stack;
			}
			var logObj = buildLogObject(message, mode, tag);

			return ZHPromise.all(
				drivers.map(function(driver) {
					return driver.save(logObj);
				})
			);
		} else {
			return ZHPromise.reject({
				err: ScreenService.i18n("Mode __mode__ not enabled.", { mode: mode })
			});
		}
	}

	function buildLogObject(message, mode, tag){
		var logObj = {};
		logObj[ZhLogger.LOG_PROPERTIES.DATE] = ((new Date()).getTime());
		logObj[ZhLogger.LOG_PROPERTIES.MESSAGE] =  message;
		logObj[ZhLogger.LOG_PROPERTIES.MODE] = mode;
		logObj[ZhLogger.LOG_PROPERTIES.TAG] = tag;
		return logObj;
	}

	this.debug = function(message, objectArray, tag) {
		return log(ZhLogger.DEBUG, message, objectArray, tag);
	};

	this.error = function(message, objectArray, tag, throwable) {
		return log(ZhLogger.ERROR, message, objectArray, tag, throwable);
	};

	this.warning = function(message, objectArray, tag) {
		return log(ZhLogger.WARNING, message, objectArray, tag);
	};

	this.info = function(message, objectArray, tag) {
		return log(ZhLogger.INFO, message, objectArray, tag);
	};

	this.fatal = function(message, objectArray, tag, throwable) {
		return log(ZhLogger.FATAL, message, objectArray, tag, throwable);
	};

	this.enableMode = function(mode) {
		if (enabledModes.indexOf(mode) === -1 && defaultModes.indexOf(mode) !== -1){
			enabledModes.push(mode);
		}
	};

	this.disableMode = function(mode) {
		var idx = enabledModes.indexOf(mode);
		if( idx !== -1){
			enabledModes.splice(idx, 1);
		}
	};

	this.find = function(dataSourceName, mode, tag, date) {
		return ZHPromise.all(
			drivers.map(function(driver) {
				return driver.find(dataSourceName, mode, tag, date);
			})
		);
	};

	this.clear = function(dataSourceName, mode) {
		return ZHPromise.all(
			drivers.map(function(driver) {
				return driver.clear(dataSourceName, mode);
			})
		);
	};

	var sendToServer = function(dataset) {
		var request = {
			"getServiceName": function() {
				return syncRoute;
			},
			"getRequestParams": function() {
				return {
					"dataset": dataset,
					"requestType": "DataSet"
				};
			},
			"serviceName": "ZhLoggerStorage/save"
		};
		return ZHPromise.all(
			requestEngine.doRequest(request)
		);
	};

	var updateSyncDate = function() {
		syncDate = new Date().getTime();
	};

	this.sync = function(dataSourceName, cls) {
		find(dataSourceName, undefined, undefined, syncDate).then(function(dataset) {
			updateSyncDate();
			return sendToServer(dataset);
		}).then(function() {
			if (cls === true) {
				clear(dataSourceName);
			}
		});
	};
	this.setLevel = function setLevel(level) {
		this.level = level;
		return this.level;
	};

}

ZhLogger.DEBUG = "debug";
ZhLogger.INFO = "info";
ZhLogger.WARNING = "warning";
ZhLogger.ERROR = "error";
ZhLogger.FATAL = "fatal";

ZhLogger.LOG_PROPERTIES = {
	DATE: "date",
	TAG: "tag",
	MODE: "mode",
	MESSAGE : "message"
};